#!/usr/bin/env python
# -*- coding: utf-8 -*-

# GIMP plugin to create a painting on satin

# (c) Dinasset 2022
#
#   History:
#
#   v1.0: 2022-01: Initial version
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import sys, os
import time
import datetime
from gimpfu import *

# define here some constant value, when required
#FixWH=3000

############################################################################
# Service routines
#---------------------------------------------------------------------------
  
#---------------------------------------- 
# for messages use: erro_msg or info_msg
#---------------------------------------- 
def erro_msg (msg):
    ###
    currenth = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(0) # box
    pdb.gimp_message ("ERROR: "+msg)
    pdb.gimp_message_set_handler(currenth) # current
    ###
def info_msg (msg):
    ###
    currenth = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(2) # error console
    pdb.gimp_message ("INFO: "+msg)
    pdb.gimp_message_set_handler(currenth) # current
    ###

#----------------------------------------------- 
# from Original drawable create the Output image
#----------------------------------------------- 
def createWrkimage(inImage, inLayer, WrkWH):
     
    owidth = inImage.width
    oheight = inImage.height
    otype = inImage.base_type
    oname = inLayer.name
    OrigWH = inLayer.width+inLayer.height    
    
   # define/calculate the scale factor for the (new) output image  
    scale = (WrkWH*1.0)/(OrigWH*1.0)      # ratio for scaling

    rwidth=int(owidth*scale)
    rheight=int(oheight*scale)
    
   # create a new image of the decided size  
   # --- copy the source layer
   # --- scale
    newImage = pdb.gimp_image_new (rwidth, rheight, otype)                    
    newBG = pdb.gimp_layer_new_from_drawable(inLayer, newImage)     # background layer
    newBG.mode = LAYER_MODE_NORMAL
    newBG.name = "NewImage: Background Layer from "+oname
    newBG.opacity = 100.0 
    newImage.add_layer(newBG, 0)
    pdb.gimp_layer_scale(newBG, rwidth, rheight, False)
    pdb.gimp_layer_add_alpha (newBG)    # RGBA
    
    # adjust mean values
    adjustmeanv (newImage, newBG)
    
   # keep a copy of the original background below the one which will be used ----------------
   # --- and set its visibility off (it should not enter in the process)
    Original = newBG.copy()
    newImage.add_layer(Original, 1)     # below
    pdb.gimp_item_set_visible (Original, False)
    Original.name = oname+" (original)"
   
    newBG.opacity = 100.0
    
    return(newImage)
#---------------------------------------------------------------------------

#---------------------------------------------------------------------------
# adjust values and saturation
#---------------------------------------------------------------------------
def adjustmeanSV (inImage, inLayer):
    # decompose HSV
    (HSVI,HL,SL,VL) = pdb.plug_in_decompose (inImage, inLayer, "HSV", True)
    #pdb.gimp_display_new (HSVI)
    HL=HSVI.layers[0]; SL=HSVI.layers[1]; VL=HSVI.layers[2];
    adjustmeanv (HSVI, SL)
    adjustmeanv (HSVI, VL)
    pdb.plug_in_recompose (HSVI, HL)
    return(inLayer)

#---------------------------------------------------------------------------
# adjust values
#---------------------------------------------------------------------------
def adjustmeanv(inImage, inLayer):
    # take care of the existing mean in determining the theoretical mean
    meanadj=50
    # check mean value of the layer
    (meanl,std,med,pix,cnt,per) = pdb.gimp_drawable_histogram (inLayer,
                                  HISTOGRAM_VALUE, 0.0, 1.0)
    thmean = (127.5*meanadj/100+meanl*(100.0-meanadj)/100)   
    #info_msg ("thmean="+str(thmean))
    # range for the loops
    minmean = thmean*1.05  
    maxmean = thmean*1.15  
    #info_msg ("target min mean value="+str(minmean))
    #info_msg ("target max mean value="+str(maxmean))
    
    if meanl<minmean:
        while meanl<minmean:
            #info_msg ("mean source value="+str(meanl))
            pdb.gimp_drawable_levels (inLayer, HISTOGRAM_VALUE,
                                      0.0, 1.0, False, 1.1,
                                      0.0, 1.0, False)
            (meanl,std,med,pix,cnt,per) = pdb.gimp_drawable_histogram (inLayer,
                                          HISTOGRAM_VALUE, 0.0, 1.0)
    elif meanl>maxmean:
        while meanl>maxmean:
            #info_msg ("mean source value="+str(meanl))
            pdb.gimp_drawable_levels (inLayer, HISTOGRAM_VALUE,
                                      0.0, 1.0, False, 0.9,
                                      0.0, 1.0, False)
            (meanl,std,med,pix,cnt,per) = pdb.gimp_drawable_histogram (inLayer,
                                          HISTOGRAM_VALUE, 0.0, 1.0)
    
    
    return(inLayer)


############################################################################
# Main
#---------------------------------------------------------------------------

def DrawTextureColoree3 (inImage, inLayer,
                   UserOutWH, NrAreas, TextureColour,
                   TextureLook, BorC, Mask, 
                   doFlatten):
    
    
    # here the logic of the filter
    oname = inLayer.name
    if "." in oname:
        split = oname.split(".", 1)
        oname = split[0]
    OrigWH = inLayer.width+inLayer.height
    
    
    ts = time.time()
    st = datetime.datetime.fromtimestamp(ts).strftime("%Y-%m-%d %H-%M-%S")
    
    # create new image to work on
    WorkWH = 1500+1500*int(TextureLook)
    newImage = createWrkimage (inImage, inLayer, WorkWH)
    width = newImage.width
    height = newImage.height
    Preprocessed = newImage.layers[0]
    Preprocessed.name = "Preprocessed"
    # simplify details thru Cartoon         <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
    # cartoon 3,200,20,0.25,1.5,11,0,50,50
    CartoonParms = "3,200,20,0.25,1.5,"+str(int(NrAreas))+",0,50,50"
    pdb.plug_in_gmic_qt(newImage, Preprocessed, 1,2,
    '-v - -cartoon '+CartoonParms+' ')
    Cartoon = newImage.layers[0]
    Cartoon.name = "Simplified"
    
    #pdb.gimp_display_new (newImage)
    #return
    
    # Paper Antique White
    Paper = pdb.gimp_layer_new (newImage, width, height, RGBA_IMAGE,
                                "Antique paper", 50.0, LAYER_MODE_MULTIPLY)  #LAYER_MODE_OVERLAY)
    newImage.add_layer (Paper, 0)
    pdb.gimp_context_set_foreground ((250,200,160,255))  #((250,185,120,255))
    pdb.gimp_edit_fill (Paper, FILL_FOREGROUND)
    
    #pdb.gimp_display_new (newImage)
    #return
    
    # Apply samj_texture_coloree
    # Set Colour (default = Sanguigna)
    (RED,GRE,BLU,A) = TextureColour  # input parameter
    #  first 0 = type A
    #  using layer scaled down half the size
    Texture = Preprocessed.copy()
    newImage.add_layer (Texture, 0)
    if BorC < 2: # option Mono(0) or Cross(1)
        pdb.gimp_layer_scale (Texture, width/2, height/2, False)
        TextureParms="0,2,"+str(RED)+","+str(GRE)+","+str(BLU)+",10,5,135,1000,4,0.2"    
    else: # option Irregular(2)
        pdb.gimp_layer_scale (Texture, width/3, height/3, False)    # strong scale down
        TextureParms="0,0.7,"+str(RED)+","+str(GRE)+","+str(BLU)+",2,2,5,200,8,0.2"    
    
    pdb.plug_in_gmic_qt(newImage, Texture, 1,0,
    '-v - -samj_texture_coloree '+TextureParms+' ')  
    Texture = newImage.layers[0]
    if BorC < 2: # option Mono(0) or Cross(1)
        Texture.mode = LAYER_MODE_MULTIPLY   
        Texture.opacity = 85.0
    else: #} option Irregular(2)
        Texture.mode = LAYER_MODE_OVERLAY   
        Texture.opacity = 45.0
    Texture.name = "samj_texture_coloree "+TextureParms
    pdb.gimp_layer_scale (Texture, width, height, False) 
    pdb.plug_in_unsharp_mask (newImage, Texture, 2.0, 1.0, 0)
    
    #pdb.gimp_display_new (newImage) 
    #return
    
    Texture2 = Preprocessed.copy()
    newImage.add_layer (Texture2, 0)
    if BorC < 2: # option Mono(0) or Cross(1)
        pdb.gimp_layer_scale (Texture2, width/2, height/2, False)
        TextureParms="0,2,"+str(RED)+","+str(GRE)+","+str(BLU)+",10,5,135,1000,4,0.2"    
    else: #} option Irregular(2)
        pdb.gimp_layer_scale (Texture2, width/3, height/3, False)    # strong scale down
        TextureParms="0,0.7,"+str(RED)+","+str(GRE)+","+str(BLU)+",2,2,5,200,8,0.2"    
    if BorC == 0: # option Mono chosen
        TextureParms="0,2,"+str(RED)+","+str(GRE)+","+str(BLU)+",10,5,135,1000,4,0.2"   
    elif BorC == 1: # option Cross chosen
        TextureParms="0,2,"+str(RED)+","+str(GRE)+","+str(BLU)+",10,5,45,1000,4,0.2"    
    else: # option Irregular chosen
        TextureParms="0,0.7,"+str(RED)+","+str(GRE)+","+str(BLU)+",2,2,175,200,8,0.2"    
    
    pdb.plug_in_gmic_qt(newImage, Texture2, 1,0,
    '-v - -samj_texture_coloree '+TextureParms+' ')  
    Texture2 = newImage.layers[0]
    if BorC == 0: # option Mono chosen
        Texture2.mode = LAYER_MODE_LINEAR_BURN
        Texture2.opacity = 85.0  
    elif BorC == 1: # option Cross chosen
        Texture2.mode = LAYER_MODE_LINEAR_BURN
        Texture2.opacity = 85.0
    else: # option Irregular chosen
        Texture2.mode = LAYER_MODE_SOFTLIGHT
        Texture2.opacity = 45.0  
    Texture2.name = "samj_texture_coloree "+TextureParms
    pdb.gimp_layer_scale (Texture2, width, height, False) 
    pdb.plug_in_unsharp_mask (newImage, Texture2, 2.0, 1.0, 0)
    
    #pdb.gimp_display_new (newImage) 
    #return

    # add samj_Posterize_B 1,7,0.1,0,10,0,0,5000,1,1,0,1,0,0,0,0,0,0  
    # activate Cartoon and Posterize it to get contours
    pdb.gimp_image_set_active_layer (newImage, Cartoon) 
    PosterParms='1,7,0.1,0,10,0,0,5000,1,1,0,1,0,0,0,0,0,0'
    pdb.plug_in_gmic_qt(newImage, Preprocessed, 1,2,
    '-v - -samj_Posterize_B '+PosterParms+' ')
    Poster = newImage.layers[0]
    Poster.mode = LAYER_MODE_LINEAR_BURN  
    Poster.opacity = 100.0  
    Poster.name = "samj_Posterize_B "+PosterParms
    
    #pdb.gimp_display_new (newImage) 
    #return
    
    # Preliminary outcome
    Preoutcome = pdb.gimp_layer_new_from_visible (newImage, newImage, "Outcome")
    newImage.add_layer (Preoutcome, 0)
    Preoutcome.name = "Preliminary Outcome (sharpened)"
    pdb.plug_in_unsharp_mask (newImage, Preoutcome, 3.0, 1.5, 0)
    
    # Remove texture from light areas using the Preprocessed (colorized) layer
    SourceOnTop = Cartoon.copy()
    newImage.add_layer (SourceOnTop, 0)
     
    SourceOnTop.name = "Simplified to overlay light areas"
    pdb.gimp_drawable_curves_spline (SourceOnTop, HISTOGRAM_VALUE,
                                     6, (0.0,0.0, 0.5,0.3, 1.0,1.0) )
    SourceOnTop.mode = LAYER_MODE_ADDITION  #GRAIN_MERGE   #LINEAR_LIGHT
    SourceOnTop.opacity = 100.0 #70.0
    LM = pdb.gimp_layer_create_mask (SourceOnTop, ADD_MASK_COPY)
    #pdb.gimp_layer_add_mask (SourceOnTop, LM)
     
    # Prep to smooth
    SOT2 = Preprocessed.copy()
    newImage.add_layer (SOT2, 0)
    pdb.gimp_drawable_desaturate (SOT2, DESATURATE_LUMA)
    SOT2.name = "Preprocessed desaturated to smooth"
    SOT2.mode = LAYER_MODE_SCREEN
    SOT2.opacity = 50.0
    
    # add a layer mask to let space free from strokes around
    (pwidth,pheight,pbpp) = pdb.gimp_pattern_get_info (Mask)
    border_layer = pdb.gimp_layer_new (newImage, pwidth, pheight, RGBA_IMAGE,
                                       "Frame "+Mask,
                                        100.0, LAYER_MODE_NORMAL)
    newImage.add_layer (border_layer, -1)
    pdb.gimp_context_set_pattern (Mask)
    pdb.gimp_drawable_fill (border_layer, FILL_PATTERN)
    pdb.gimp_layer_scale (border_layer, width, height, False)     
    pdb.gimp_context_set_foreground ((200,180,150,255))  #((250,185,120,255))
    pdb.gimp_image_select_contiguous_color (newImage, CHANNEL_OP_REPLACE,
                                            border_layer, 1.0, 1.0)
    pdb.gimp_edit_fill (border_layer, FILL_FOREGROUND)
    pdb.gimp_selection_none (newImage)
    
    # final step
    if doFlatten==True:
        outcome = pdb.gimp_image_merge_visible_layers (newImage, CLIP_TO_BOTTOM_LAYER)
        outcome.name = "Outcome" 
    else:    
        outcome = pdb.gimp_layer_new_from_visible (newImage, newImage, "Outcome")
        newImage.add_layer (outcome, 0)
        pdb.gimp_item_set_visible (outcome, False)
        
    outcome.name = st+oname+" as a drawing with texture coloree (3)"
    options = ["basic", "regular", "fine" ]
    optionused = " (lines look "+options[TextureLook]+")"
    outcome.name = outcome.name+optionused
    options2 = ["mono", "cross", "undefined"]
    option2used = " (lines look 2 "+options2[BorC]+")"
    outcome.name = outcome.name+option2used
    colorused = " (texture colour used ["+str(RED)+","+str(GRE)+","+str(BLU)+"])"
    outcome.name = outcome.name+colorused
    areasused = " (nr of areas "+str(int(NrAreas))+")"
    outcome.name = outcome.name+areasused
    
    # user required output image size
    if UserOutWH==0: UserOutWH=OrigWH
    if UserOutWH != WorkWH:
        uRatio = UserOutWH*1.00/WorkWH*1.00
        uWidth = int(width*uRatio)
        uHeight = int(height*uRatio)
        #info_msg ("SCALING")   # not during tests, scaling is slow
        pdb.gimp_image_scale (newImage, uWidth, uHeight)   # to user chosen size
    
    pdb.gimp_display_new (newImage)

### Registration
whoiam="\n"+os.path.abspath(sys.argv[0])
author="dinasset"
menu="<Image>/Diego/SimpleGraphics/"
copyrightYear="2022"
desc="Draw Texture Coloree (3)"

register(
    "DrawTextureColoree3",
    desc+whoiam,desc,
    author,author,
    copyrightYear,
    desc+" ...",
    "*",
    [
        (PF_IMAGE, "image", "Input image", None),
        (PF_DRAWABLE, "drawable", "Input drawable", None),
        (PF_ADJUSTMENT, "newsize", "Width+Height of New Image"+
                                   "\n (0=same as input)",
                                   3000, (0, 7200, 100)),
        (PF_ADJUSTMENT, "areas", "Nr of colour areas to simplify image",
                                   15, (5, 255, 1)),
        (PF_COLOR,  "color", "Texture colour", (125,125,125)),     
        (PF_OPTION, "lines", "Lines look", 1, ["Basic", "Regular", "Fine" ]),
        (PF_OPTION, "lines2", "Lines look 2", 1, ["Monodirectional", "Crossing", "Undefined" ]),
        (PF_PATTERN, "maskpat", "Pattern for mask", "Mask_Grunge_10.png"),
        (PF_TOGGLE, "flatten", "Flatten Image?", False),
    ],
    [],
    DrawTextureColoree3,
    menu=menu
)

main()
